'use strict';

var path = require('path');
var fs = require('fs');
var ariaMocha = require('aria-mocha');
var esbuild = require('esbuild');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);

const patterns = () => [
  "**/*.spec.ts",
  "**/*.spec.js",
  "**/*.spec.jsx",
  "**/*.spec.tsx",
  "**/*.test.ts",
  "**/*.test.js",
  "**/*.test.jsx",
  "**/*.test.tsx"
];
const createUrl = ({port, hostname, path: path2}) => `http://${hostname}:${port}/${path2}`;
async function launchHeadless({hostname, port, path: path2, watch}) {
  await ariaMocha.launch(createUrl({hostname, port, path: path2}));
  !watch && process.exit();
}
async function getFiles(file, relative) {
  const stat = await fs.promises.lstat(file);
  return stat.isDirectory() ? await ariaMocha.globFiles(patterns().map((pattern) => path__default['default'].join(file, pattern)), relative) : stat.isFile() ? [file] : [];
}
function configureServer(options) {
  const {dir = "tests", watch = false} = options || {};
  let port = 3e3, hostname = "localhost", indexPath = "node_modules/vite-plugin-test/index.html";
  return (server) => {
    var _a;
    server.middlewares.use("/test-files", async (_req, res, _next) => {
      const files = fs.existsSync(dir) ? await getFiles(dir, true) : [];
      res.writeHead(200, {
        "Content-type": "application/json"
      });
      res.end(JSON.stringify(files));
    });
    server.watcher.on("change", async (path2, stats) => {
      await launchHeadless({hostname, port, watch, path: indexPath});
    });
    (_a = server.httpServer) == null ? void 0 : _a.on("listening", async () => {
      var _a2;
      const address = (_a2 = server.httpServer) == null ? void 0 : _a2.address();
      hostname = address.address === "::" ? "localhost" : address.address;
      port = address.port;
      await launchHeadless({hostname, port, watch, path: indexPath});
    });
  };
}

const getExt = (str) => {
  const basename = path__default['default'].basename(str);
  const firstDot = basename.indexOf(".");
  const lastDot = basename.lastIndexOf(".");
  const extname = path__default['default'].extname(basename).replace(/(\.[a-z0-9]+).*/i, "$1");
  if (firstDot === lastDot)
    return extname;
  return basename.slice(firstDot, lastDot) + extname;
};
function transform(options) {
  const {loaders} = options || {};
  return (code, id) => {
    const ext = getExt(id);
    if (loaders && Object.keys(loaders).includes(ext)) {
      const loader = (options == null ? void 0 : options.loaders) && (options == null ? void 0 : options.loaders[ext]) ? options.loaders[ext] : path__default['default'].extname(id).slice(1);
      const result = esbuild.transformSync(code, {
        loader,
        format: "esm",
        sourcemap: false,
        target: "es2017"
      });
      return {
        code: result.code
      };
    }
    return {code};
  };
}

function viteTestPlugin(options) {
  const plugin = {
    name: "vite-plugin-test",
    apply: "serve",
    configureServer: configureServer(options),
    transform: transform(options)
  };
  return plugin;
}

module.exports = viteTestPlugin;
